
; Filename: Thermometer12F509 .ASM
; **********************************************
; * Author:   Franois Callanquin              *
; * Revision: 0.0                              *
; * Date      November 15, 2014                *
; * Part:     PIC12F675                        *
; * Compiled using MPLAB IDE V8.92             *
; **********************************************


        ; use PIC 12F675
        list                p=12f675
        #include        <p12f675.inc>

        ; set configuration word.

 ;       __CONFIG _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_ON & _PWRTE_ON & _INTRC_OSC_NOCLKOUT
       __CONFIG _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_ON & _PWRTE_OFF & _INTRC_OSC_NOCLKOUT

; ************************************
; * Port (GPIO) bit definitions      *
; ************************************
#define Transmit	GPIO,2
#define Power		GPIO,5
#define SCL			GPIO,1
#define SDA			GPIO,4


; transmit 2.34s
; get battery voltage on AN0 : duration 32us 

; init + wait + read DS1721 18mA : 156ms

; transmit RF 43mA :1.74s - 156ms (5 times)
; transmit RF 43mA :800ms - 156ms (2 times)

; uncomment device you need to compile
 #define DEVICE_0
; #define DEVICE_1
; #define DEVICE_2
; #define DEVICE_3
; #define DEVICE_4
; #define DEVICE_5
; #define DEVICE_6
; #define DEVICE_7



#IFDEF DEVICE_0 
	#define DEVICE_ID	0x00	; device 0 
	#define INTERVALL  	.240	; 8mn intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_1 
	#define DEVICE_ID	0x20	; device 1
	#define INTERVALL  	.238	; 7mn  56s sintervall before two wake up and measurements = value x 2 seconds 
#ENDIF
#IFDEF DEVICE_2 
	#define DEVICE_ID	0x40	; device 2
	#define INTERVALL  	.236	; 7mn 52s intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_3 
	#define DEVICE_ID	0x60	; device 3
	#define INTERVALL  	.234	; 7mn 48s intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_4
	#define DEVICE_ID	0x80	; device 4
	#define INTERVALL  	.232	; 7mn 44s intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_5
	#define DEVICE_ID	0xA0	; device 5
	#define INTERVALL  	.230	; 7mn 40s intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_6
	#define DEVICE_ID	0xC0	; device 6
	#define INTERVALL  	.228	; 7mn 36s intervall before two wake up and measurements = value x 2 seconds
#ENDIF
#IFDEF DEVICE_7
	#define DEVICE_ID	0xE0	; device 7
	#define INTERVALL  	.226	; 7mn 32s intervall before two wake up and measurements = value x 2 seconds
#ENDIF


; **************
; * I2c macros *
; **************
SetSCL	macro
        call MS2
        bsf SCL
        call MS2
		endm

ClrSCL	macro
        call MS2
        bcf SCL
        call MS2
		endm

SetSDA  macro
        call MS2
        bsf SDA
        call MS2
		endm

ClrSDA	macro
       call MS2
       bcf SDA
       call MS2
		endm



; ********************************************
; * The PC starts at the Beginning of memory *
; ********************************************
    ORG     0h

        bsf STATUS, RP0 ;Bank 1                ; force OSCCAL
        call 3FFh ;Get the cal value
        movwf OSCCAL ;Calibrate
        bcf STATUS, RP0 ;Bank 0

;-------------- INITIALISE PORTS --------------------------------------------------

        bcf     STATUS,RP0  ;Bank 0
        clrwdt              ;Clear WDT
        clrf    TMR0        ;Clear TMR0 and  prescaler
     	
		movlw	b'00000111'	; comparator off
 		movwf	CMCON   
		
		movlw	b'00000001'	; AD converter is operating 
		movwf	ADCON0	

		bsf     STATUS,RP0   ;Bank 1

        movlw   b'00101111' ; Required if desired
        movwf   OPTION_REG  ; PS2S0 is
		clrwdt              ; 000 or 001

        movlw   b'00101111' ; Set postscaler to
        movwf   OPTION_REG  ; desired WDT rate
		
		movlw	b'00010001' ; GPIO0 as analog input Fosc /8
		movwf   ANSEL

      	movlw	b'00001111' 
		movwf	VRCON	  
		movlw   0x01         ; GPIO  output
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0


        movfw STATUS
        btfss STATUS,4
              goto Sleep_mode

        goto loop


 
;****************************
; INTERNAL VARIABLES
;****************************

CURRENT_INPUT	  EQU 0x20
TEMP              EQU 0x21
COUNT1            EQU 0x22
COUNT2            EQU 0x23
Read_I2C_count    EQU 0x24
I2C_byte_MSB      EQU 0x25
I2C_byte_LSB      EQU 0x26
DIGIT_1           EQU 0x27
DIGIT_2           EQU 0x28
DIGIT_3           EQU 0x29
CRC               EQU 0x2A
COUNT3            EQU 0x2B
COUNT4            EQU 0x2C
COUNT5            EQU 0x2D
RS232_byte        EQU 0x2E
Read_RS232_count  EQU 0x2F
an0_temp          EQU 0x30        
status_temp       EQU 0x31        ; used for interrupt
I2C_byte          EQU 0x32
sleep_cnt         EQU 0x33


;**************************
; Routines in first page  *
;**************************
;**********************************************;
; SET TRANSMIT bit ONE (1.5ms ON / 0.5ms OFF) *;
;**********************************************;
SetTransmit:
		bsf 	Transmit
		MOVLW 	0x0BB		; 187 * 8 cycles = 1496 us
		MOVWF   COUNT1                 
loop_1500u_S:               ; loop 8 cycles
		nop
		nop
		nop
		nop
		nop
		DECFSZ  COUNT1,1    ; 1 cycles when not 0 , 1 cycles when 1
		GOTO   	loop_1500u_S; 2 cycles
		nop
		nop
		
		bcf 	Transmit
		MOVLW   0x03E         ; 62 * 8 cycles = 496 us
		MOVWF   COUNT1                 
loop_500u_S:                  ; loop 8 cycles
		nop
		nop
		nop
		nop
		nop
		DECFSZ	COUNT1,1      ; 1 cycles when not 0 , 1 cycles when 1
		GOTO    loop_500u_S	  ; 2 cycles
		nop
		nop
		retlw 0 
;**********************************************;
; CLR TRANSMIT bit SERO (0.5ms ON / 1.5ms OFF)    *;
;**********************************************;
ClrTransmit:
         bsf 	Transmit
         MOVLW  0x03E          ; 62 * 8 cycles = 496 us
         MOVWF  COUNT1                 
loop_500u_C:                  ; loop 8 cycles
         nop
         nop
         nop
         nop
         nop
         DECFSZ	COUNT1,1       ; 1 cycles when not 0 , 1 cycles when 1
         GOTO   loop_500u_C    ; 2 cycles
         bcf    Transmit
         MOVLW  	0x0BB	   ; 187 * 8 cycles = 1496 us
         MOVWF  COUNT1                 
loop_1500u_D:                  ; loop 8 cycles
         nop
         nop
         nop
         nop
         nop
         DECFSZ	COUNT1,1       ; 1 cycles when not 0 , 1 cycles when 1
         GOTO   loop_1500u_D   ; 2 cycles
         retlw 0 


;**************************************************
; Send byte I2C
; byte to write in W 
;**************************************************

Send_Byte_I2C: 

        movwf I2C_byte

        movlw 0x80
        movwf Read_I2C_count


write_I2C_loop:
        ClrSCL 
        SetSDA                   ; prepare SDA
        movf  I2C_byte,w
        andwf Read_I2C_count,w

        BTFSS STATUS,Z
              goto not_clr_sda
        ClrSDA
not_clr_sda:

         bcf STATUS,0            ; clear Carry
         rrf Read_I2C_count,f    ; shift count
         SetSCL 

         movf 	Read_I2C_count,w
         btfss	STATUS,Z
         goto 	write_I2C_loop

        retlw 0 

 ; transmit one on I2C
Transmit_ONE:
        ClrSCL ;
        SetSDA ;        
        SetSCL ;        // 1
        retlw 0 

; transmit one on I2C
Transmit_ZERO:

        ClrSCL ;        
        ClrSDA ;        
        SetSCL ;        // 0
        retlw 0 

; transmit Write on I2C
Transmit_WRITE:
        ClrSCL ;
        ClrSDA ;        
        SetSCL ;        // write
        retlw 0 

; transmit ACK on I2C
Transmit_ACK:
        ClrSCL ;
        SetSDA ;        
        SetSCL ;        // ACK
        retlw 0 


;*********** DELAY 8.5us  ***********************
MS2:        MOVLW	0x01          	; 1 cycle
            MOVWF   COUNT1          ; 1 cycle
LP15        MOVLW   0x01            ; 1 cycle
            MOVWF   COUNT2          ; 1 cycle

LP16        DECFSZ  COUNT2,1        ; 2 cycles when not 0 , 1 cycles when 1
                GOTO        LP16    ; 2 cycles

            DECFSZ  COUNT1,1  		; 2 cycles when not 0 , 1 cycles when 1
            GOTO    LP15            ; 2 cycles
            RETLW   0           

;**************************************************
; Send byte RS232
; byte to write in W 
;**************************************************

Send_Byte_RS232: 

        clrwdt                        ;Clear WDT

        movwf RS232_byte
		bcf 	Transmit

        ; 50ms interchar
        MOVLW   .16        				; no modulation in idle ..
        MOVWF   COUNT3

loop_RST:
        MOVLW	0x0BB                   ; 187 * 8 cycles = 1496 us
        MOVWF   COUNT1                 
loop_1500u_I:                          	; loop 8 cycles
        nop
        nop
        nop
        nop
        nop
        DECFSZ  COUNT1,1                ; 2 cycles when not 0 , 1 cycles when 1
        GOTO    loop_1500u_I        	; 2 cycles
        DECFSZ  COUNT3,1
        GOTO 	loop_RST

        movlw 0x01
        movwf Read_RS232_count
        
        call ClrTransmit              ; START x 4 for receiver sync
        call ClrTransmit
        call ClrTransmit
        call ClrTransmit

        call         SetTransmit     	; force one for sync

write_RS232_loop:
        movf	RS232_byte,w
        andwf   Read_RS232_count,w

        btfsc   STATUS,Z
               	goto        ClrTransmit_BYTE
        call    SetTransmit
        goto continue
ClrTransmit_BYTE:
        call    ClrTransmit
continue:
	      bcf 	STATUS,0               ; clear Carry
	      rlf 	Read_RS232_count,f     ; shift count
	      
	      movf 	Read_RS232_count,w
	      btfss	STATUS,Z
	            goto write_RS232_loop
	
	      call SetTransmit             ; STOP
	      bcf 	Transmit
	
end_tx:
        retlw 0 
  

;********* MAIN PROG ************
;org 0x100
loop:
        clrf	sleep_cnt

		; get battery voltage on AN0 , duration 32us 
		;===========================================
		bsf	ADCON0,1			; start convertion
wait_adc:
		BTFSC ADCON0,1
			goto wait_adc
    
		bcf     Power      ; set power on HF and I2C Thermometer

 ;       bsf Transmit 


;***********************************************************
; Write configuration register and start convertion I2C on *
;***********************************************************
Start_I2C:

        CLRWDT                        ; reset watchdog

; WRITE CONFIG REGISTER
        SetSDA        ;  SDL =1 & SDA = 1
        SetSCL        

; transmit START on I2C
		clrwdt                        ; Clear WDT
        call MS2
        SetSCL
        call MS2
        call MS2
        call MS2
        ClrSDA

        call MS2

        ;  0x4F address
        movlw 0x9E                   ; 4F + write
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ;  0xAC config 
        movlw 0xAC
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ; write 0x04 into config register (10 bits resolution = 0.25C)
 ;       movlw 0x04
 ; write 0x04 into config register (10 bits resolution = 0.25C) and one shot
         movlw 0x05
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ClrSCL ;
        SetSCL ;
        SetSDA ; stop

; WRITE START CONVERTION COMMAND
        SetSCL        				; SDL =1 & SDA = 1
        SetSDA
                
; transmit START on I2C
        call MS2
        SetSCL
        call MS2
        ClrSDA
        call MS2

        ;  0x4F address
        movlw 0x9E                  ; 4F + write
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ; DS1721 0xEE config 
        movlw 0x51                
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ClrSCL ;
        SetSCL ;
        SetSDA ;        // stop




;**************************************************
; wait 250ms for conversion done (187ms required) *
;**************************************************
Wait_I2C_Thermometer_convertion_done:
        CLRWDT        ; reset watchdog

        SetSCL        ; SDL =1 & SDA = 1
        SetSDA

; transmit START on I2C
        call MS2
        SetSCL
        call MS2
        ClrSDA
        call MS2

        ;  0x4F address
        movlw 0x9E                        ; 4F + write
        call Send_Byte_I2C
        call Transmit_ACK ; ACK
                
        ;  DS1721 0xAC read config register command 
        movlw 0xAC                
        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ClrSCL 		; restart
        SetSDA
        SetSCL
        ClrSCL 		; restart

; transmit START on I2C
        call MS2
        SetSCL
        call MS2
        ClrSDA
        call MS2

        ;  0x4F address
        movlw 0x9F                        ; 4F + read
        call Send_Byte_I2C

        call Transmit_ACK ; ACK

        movlw 0x08
        movwf Read_I2C_count
        clrf I2C_byte_MSB

read_I2C_loop_WAIT:
        ClrSCL ;
        SetSDA ;        
        SetSCL ;        // ACK

		bsf		STATUS,RP0   ;Bank 1
		movlw   0x11        ; GPIO SDA input
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0
		nop
        bcf STATUS,0        ; clear Carry
        btfsc  SDA          ; read SDA on port B
        bsf STATUS,0    	; set carry
        rlf I2C_byte_MSB,f  ;rotate left with carry

        DECFSZ	Read_I2C_count,1
        goto 	read_I2C_loop_WAIT

		bsf     STATUS,RP0   ;Bank 1
		movlw   0x01         ; GPIO SDA output
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0
		nop
        call Transmit_ACK ; ACK

        SetSCL 
        SetSDA 				; stop

        btfss   I2C_byte_MSB, 7	; check convertion done
        	    goto Wait_I2C_Thermometer_convertion_done

;*******************************
; Get input I2C on Thermometer *
;*******************************
Read_I2C_Thermometer:

        CLRWDT        ; reset watchdog

        SetSCL        ; SDL =1 & SDA = 1
        SetSDA

; transmit START on I2C
        call MS2
        SetSCL
        call MS2
        ClrSDA
        call MS2

        ;  0x4F address
        movlw 0x9E                        ; 4F + write
        call Send_Byte_I2C
        call Transmit_ACK ; ACK
                
        ;  DS1721 0xAA read command 
        movlw 0xAA                

        call Send_Byte_I2C
        call Transmit_ACK ; ACK

        ClrSCL 		; restart
        SetSDA
        SetSCL
        ClrSCL 		; restart

; transmit START on I2C
        call MS2
        SetSCL
        call MS2
        ClrSDA
        call MS2

        ;  0x4F address
        movlw 0x9F                        ; 4F + read
        call Send_Byte_I2C

        call Transmit_ACK ; ACK

        movlw 0x08
        movwf Read_I2C_count
        clrf I2C_byte_MSB

read_I2C_loop_MSB:
        ClrSCL ;
        SetSDA ;        
        SetSCL ;        // ACK

		bsf		STATUS,RP0   ;Bank 1
		movlw   0x11        ; GPIO SDA input
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0
		nop
        bcf STATUS,0        ; clear Carry
        btfsc  SDA          ; read SDA on port B
        bsf STATUS,0    	; set carry
        rlf I2C_byte_MSB,f  ;rotate left with carry

        DECFSZ	Read_I2C_count,1
        goto 	read_I2C_loop_MSB

		bsf     STATUS,RP0   ;Bank 1
		movlw   0x01         ; GPIO SDA output
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0
		nop

        call Transmit_ZERO ; master ACK

        movlw 0x08
        movwf Read_I2C_count
        clrf I2C_byte_LSB

read_I2C_loop_LSB:
        ClrSCL 
        SetSDA         
        SetSCL 					; ACK

		bsf		STATUS,RP0   	 ;Bank 1
		movlw   0x11        	 ; GPIO SDA input
        movwf	TRISIO
        bcf     STATUS,RP0   	 ;Bank 0
		nop
        bcf 	STATUS,0    	 ; clear Carry
        btfsc   SDA      		 ; read SDA on port B
        bsf 	STATUS,0   		 ; set carry
        rlf 	I2C_byte_LSB,f   ;rotate left with carry

        DECFSZ  Read_I2C_count,1
        goto 	read_I2C_loop_LSB

		bsf     STATUS,RP0   ;Bank 1
		movlw   0x01        ; GPIO SDA output
        movwf	TRISIO
        bcf     STATUS,RP0   ;Bank 0
		nop

        call Transmit_ZERO ; master ACK

        SetSCL 
        SetSDA 				; stop



        ; Byte ID :
        ; device ID b7,b6,b5 = 001
        ; battery LSB  b4,b3
        ; temperature decimal bit b2 b1 b0  

 		
		movf 	ADRESH,w
		MOVWF   DIGIT_3			; convertion MSB in DIGIT_3

        ; write byte ID
        MOVLW  DEVICE_ID 
		MOVWF	DIGIT_1

		; add analog LSB
		bsf 	STATUS, RP0 ;Bank 1 
        movf 	ADRESL,w
 		bcf 	STATUS, RP0 ;Bank 0 
		MOVWF	an0_temp
		bcf 	STATUS,0        ; clear Carry
        rrf 	an0_temp,f  ; rotate right with carry
        rrf 	an0_temp,f  ; rotate right with carry
        rrf 	an0_temp,f  ; rotate right with carry
        movf 	an0_temp,w	
		andlw	0x1C


        addwf 	DIGIT_1,w	; add analog convertion (b4,b3) with device ID (b7,b6,b5)
        MOVWF	DIGIT_1

       ; get temperature decimals
        movf 	I2C_byte_LSB,w
        bcf 	STATUS,0        ; clear Carry
        rrf 	I2C_byte_LSB,f  ; rotate right with carry
        rrf 	I2C_byte_LSB,f  ; rotate right with carry
        rrf 	I2C_byte_LSB,f  ; rotate right with carry
        rrf 	I2C_byte_LSB,f  ; rotate right with carry
        rrf 	I2C_byte_LSB,f  ; rotate right with carry
	   	movf	I2C_byte_LSB,w
		addwf 	DIGIT_1,w		; add temperature decimal b2,b1,b0
		MOVWF	DIGIT_1
	
        movf	I2C_byte_MSB,w	; temperature MSB
        MOVWF	DIGIT_2

        ; compute CRC
        movf    DIGIT_1,w
        addwf  	DIGIT_2,w
        addwf   DIGIT_3,w
        movwf   CRC
        MOVLW   0x02
        MOVWF   COUNT5

 
	    movlw	0xAA
        call	Send_Byte_RS232

retransmit_loop:

        movlw	0xAA
        call	Send_Byte_RS232
        movlw	0x55
        call	Send_Byte_RS232
        movfw	DIGIT_1
        call	Send_Byte_RS232
        movfw	DIGIT_2        
        call	Send_Byte_RS232
        movfw	DIGIT_3        
        call	Send_Byte_RS232
        movfw	CRC
        call	Send_Byte_RS232

        DECFSZ  COUNT5,1
                GOTO retransmit_loop

        movlw   B'00100000' 
        movwf   GPIO


sleep_loop                          ; Sleep mode
        clrwdt
        sleep
        nop

Sleep_mode:        
        incf	sleep_cnt,f
        movlw	INTERVALL           ; time bewteen measurements 

        subwf	sleep_cnt, W
        btfss   STATUS, Z
        	    goto sleep_loop
        clrf    sleep_cnt

        movlw        0x21           ; GPIO AN0 input output
		movf         GPIO,w

        goto loop





    END